@@ -1,6 +1,6 @@
 package Sidef::Optimizer {
 
-    use 5.016;
+    use 5.014;
     use Scalar::Util qw(refaddr);
 
     use constant {
@@ -9,9 +9,6 @@ package Sidef::Optimizer {
                   REGEX       => 'Sidef::Types::Regex::Regex',
                   BOOL        => 'Sidef::Types::Bool::Bool',
                   ARRAY       => 'Sidef::Types::Array::Array',
-                  RANGENUM    => 'Sidef::Types::Range::RangeNumber',
-                  DIR_DT      => 'Sidef::DataTypes::Glob::Dir',
-                  FILE_DT     => 'Sidef::DataTypes::Glob::File',
                   NUMBER_DT   => 'Sidef::DataTypes::Number::Number',
                   STRING_DT   => 'Sidef::DataTypes::String::String',
                   COMPLEX_DT  => 'Sidef::DataTypes::Number::Complex',
@@ -20,29 +17,6 @@ package Sidef::Optimizer {
                   BACKTICK_DT => 'Sidef::DataTypes::Glob::Backtick',
                  };
 
-    my %dt_table = (
-        qw(
-          Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
-          Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
-          Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
-          Sidef::DataTypes::Array::Matrix         Sidef::Types::Array::Matrix
-          Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
-          Sidef::DataTypes::Set::Set              Sidef::Types::Set::Set
-          Sidef::DataTypes::Set::Bag              Sidef::Types::Set::Bag
-          Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
-          Sidef::DataTypes::String::String        Sidef::Types::String::String
-          Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
-          Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
-          Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
-          Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
-          Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
-          Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
-          Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
-          Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
-          Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
-          )
-    );
-
     my %cache;
     {
 
@@ -55,26 +29,53 @@ package Sidef::Optimizer {
                     defined(my $method = UNIVERSAL::can($package, $_))
                       or die "[ERROR] Invalid method $package: $_";
                     $method;
-                }
+                  }
             } @names;
         }
 
+        my %table = (
+            qw(
+              Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
+              Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
+              Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
+              Sidef::DataTypes::Array::MultiArray     Sidef::Types::Array::MultiArray
+              Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
+              Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
+              Sidef::DataTypes::String::String        Sidef::Types::String::String
+              Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
+              Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
+              Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
+              Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
+              Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
+              Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
+              Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
+              Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
+              Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
+              )
+        );
+
+        my %seen;
+
         sub dtypes {
             my ($type, @names) = @_;
 
-            exists($dt_table{$type}) || die "[ERROR] Non-existent data type: $type";
-
-            my $package = $dt_table{$type};
-            my $module  = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
+            exists($table{$type}) || die "[ERROR] Non-existent data type: $type";
 
+            my $package = $table{$type};
+            my $module = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
             exists($INC{$module}) || require($module);
 
+            if (not $seen{$type}++) {
+                no strict 'refs';
+                push @{$type . '::' . 'ISA'}, $package;
+            }
+
             map {
                 $cache{$type, $_} //= do {
-                    defined(my $method = UNIVERSAL::can($package, $_))
-                      or die "[ERROR] Invalid method $package: $_";
+                    defined(my $method = UNIVERSAL::can($type, $_))
+                      or die "[ERROR] Invalid method $type: $_";
                     $method;
-                }
+                  }
             } @names;
         }
     }
@@ -279,23 +280,14 @@ package Sidef::Optimizer {
                    eq ne
                    and or xor
 
-                   sigma
-                   usigma
-
-                   prime_sigma
-                   prime_usigma
-
-                   is_power
-
                    complex
                    root iroot
                    log
                    next_pow
                    max min
                    roundf
-                   binomial
-                   subfactorial
-                   invmod
+                   nok
+                   modinv
                    isub
                    iadd
                    imul
@@ -319,14 +311,9 @@ package Sidef::Optimizer {
                    inc dec not
 
                    factorial
-                   subfactorial
-                   superfactorial
-                   hyperfactorial
-                   lnsuperfactorial
-                   lnhyperfactorial
                    sqrt isqrt
                    next_pow2
-                   abs int rat float complex
+                   abs int rat float i
                    norm conj sqr
 
                    zeta
@@ -338,21 +325,6 @@ package Sidef::Optimizer {
                    Li2
                    Ei
 
-                   sigma
-                   sigma0
-
-                   usigma
-                   usigma0
-
-                   prime_sigma
-                   prime_usigma
-
-                   divisors
-                   udivisors
-
-                   factor
-                   factor_exp
-
                    exp
                    exp2
                    exp10
@@ -404,11 +376,13 @@ package Sidef::Optimizer {
                    sign
                    nan
                    chr
-
-                   i pi tau
-                   e ln2 phi
-                   EulerGamma
-                   CatalanG
+                   pi
+                   ln2
+                   phi
+                   tau
+                   e
+                   euler
+                   catalan
 
                    is_zero
                    is_one
@@ -423,7 +397,6 @@ package Sidef::Optimizer {
 
                    is_prime
                    is_square
-                   is_power
 
                    sgn
                    ceil
@@ -457,7 +430,7 @@ package Sidef::Optimizer {
             (
              map { [$_, [table(NUMBER), table(NUMBER)]] }
                methods(NUMBER, qw(
-                   powmod
+                   modpow
                    range
                    )
                )
@@ -486,7 +459,6 @@ package Sidef::Optimizer {
                    first
                    last
 
-                   freq
                    len end
                    is_empty
                    min max
@@ -494,12 +466,6 @@ package Sidef::Optimizer {
                    sum
                    prod
 
-                   zip
-                   cartesian
-                   permutations
-                   derangements
-                   circular_permutations
-
                    sort
                    reverse
 
@@ -522,15 +488,7 @@ package Sidef::Optimizer {
                    exists
                    defined
                    contains
-
-                   subsets
-                   partitions
-                   variations
-                   variations_with_repetition
-                   combinations
-                   combinations_with_repetition
-
-                   nth_permutation
+                   contains_type
 
                    div
                    mul
@@ -560,6 +518,7 @@ package Sidef::Optimizer {
                    concat
 
                    eq ne
+                   mzip
                    contains_type
                    contains_any
                    contains_all
@@ -586,30 +545,6 @@ package Sidef::Optimizer {
             (map { [$_, [table('')]] } methods(ARRAY, qw(reduce_operator))),
                              ),
 
-        (RANGENUM) => build_tree(
-
-            # RangeNum.method(Number)
-            (
-             map { [$_, [table(NUMBER)]] }
-               methods(RANGENUM, qw(
-                   from to by
-                   sum add sub mul div
-                   )
-               )
-            ),
-
-            # RangeNum.method()
-            (
-             map { [$_, []] }
-               methods(RANGENUM, qw(
-                   first last reverse
-                   min max step
-                   sum length
-                   )
-               )
-            ),
-        ),
-
         (NUMBER_DT) => build_tree(
 
             # Number.method()
@@ -619,9 +554,9 @@ package Sidef::Optimizer {
                    pi
                    tau
                    ln2
-                   EulerGamma
-                   CatalanG
-                   e i
+                   euler
+                   catalan
+                   e
                    phi
                    nan
                    inf
@@ -682,30 +617,6 @@ package Sidef::Optimizer {
             ),
         ),
 
-        (FILE_DT) => build_tree(
-
-            # File.method(STRING)
-            (
-             map { [$_, [table(STRING)]] }
-               dtypes(FILE_DT, qw(
-                   new
-                   )
-               )
-            ),
-        ),
-
-        (DIR_DT) => build_tree(
-
-            # Dir.method(STRING)
-            (
-             map { [$_, [table(STRING)]] }
-               dtypes(DIR_DT, qw(
-                   new
-                   )
-               )
-            ),
-        ),
-
         (RANGENUM_DT) => build_tree(
 
             # RangeNum.method(NUMBER, NUMBER)
@@ -717,7 +628,7 @@ package Sidef::Optimizer {
                )
             ),
 
-            # RangeNum.method(NUMBER, NUMBER, NUMBER)
+            # RangeNum.method(NUMBER, NUMBER)
             (
              map { [$_, [table(NUMBER), table(NUMBER), table(NUMBER)]] }
                dtypes(RANGENUM_DT, qw(
@@ -753,7 +664,31 @@ package Sidef::Optimizer {
 
             # Complex.method(NUMBER|STRING, NUMBER|STRING)
             (
-             map { [$_, [table(STRING, NUMBER), table(STRING, NUMBER)]] }
+             map { [$_, [table(STRING), table(STRING)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(STRING), table(NUMBER)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(NUMBER), table(STRING)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(NUMBER), table(NUMBER)]] }
                dtypes(COMPLEX_DT, qw(
                    new
                    )
@@ -792,7 +727,7 @@ package Sidef::Optimizer {
         if ($ref eq 'HASH') {
             $obj = $self->optimize($obj);
         }
-        elsif ($ref eq 'Sidef::Variable::Variable') {
+        elsif ($ref eq "Sidef::Variable::Variable") {
             if ($obj->{type} eq 'var') {
                 ## ok
             }
@@ -805,19 +740,13 @@ package Sidef::Optimizer {
                 }
             }
         }
-        elsif (
-               (
-                   $ref eq 'Sidef::Variable::Static'
-                or $ref eq 'Sidef::Variable::Const'
-                or $ref eq 'Sidef::Variable::Define'
-               )
-               and exists($obj->{expr})
-          ) {
+        elsif ($ref eq 'Sidef::Variable::Static') {
             if ($addr{refaddr($obj)}++) {
                 ## ok
             }
             else {
-                $obj->{expr} = {$self->optimize($obj->{expr})};
+                my %code = $self->optimize($obj->{expr});
+                $obj->{expr} = \%code;
             }
         }
         elsif ($ref eq 'Sidef::Variable::Init') {
@@ -826,7 +755,8 @@ package Sidef::Optimizer {
             }
             else {
                 if (exists $obj->{args}) {
-                    $obj->{args} = {$self->optimize($obj->{args})};
+                    my %code = $self->optimize($obj->{args});
+                    $obj->{args} = \%code;
                 }
             }
         }
@@ -841,16 +771,8 @@ package Sidef::Optimizer {
                 ## ok
             }
             else {
-                $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
-            }
-        }
-        elsif ($ref eq 'Sidef::Types::Block::Try') {
-            if ($addr{refaddr($obj)}++) {
-                ## ok
-            }
-            else {
-                $obj->{try}{code}   = {$self->optimize($obj->{try}{code})};
-                $obj->{catch}{code} = {$self->optimize($obj->{catch}{code})} if defined($obj->{catch});
+                my %code = $self->optimize($obj->{block}{code});
+                $obj->{block}{code} = \%code;
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
@@ -858,7 +780,8 @@ package Sidef::Optimizer {
                 ## ok
             }
             else {
-                $obj->{code} = {$self->optimize($obj->{code})};
+                my %code = $self->optimize($obj->{code});
+                $obj->{code} = \%code;
             }
         }
         elsif ($ref eq 'Sidef::Types::Array::HCArray') {
@@ -875,67 +798,60 @@ package Sidef::Optimizer {
 
             # Has no expressions, so let's convert it into an Array
             #~ if (not $has_expr) {
+
             #~ #$obj = Sidef::Types::Array::Array->new(@{$obj});
             #~ bless $obj, 'Sidef::Types::Array::Array';
             #~ }
         }
         elsif ($ref eq 'Sidef::Types::Block::If') {
             foreach my $i (0 .. $#{$obj->{if}}) {
-                $obj->{if}[$i]{block}{code} = {$self->optimize($obj->{if}[$i]{block}{code})};
-                $obj->{if}[$i]{expr} = {$self->optimize($obj->{if}[$i]{expr})};
+                my %code = $self->optimize($obj->{if}[$i]{block}{code});
+                $obj->{if}[$i]{block}{code} = \%code;
+
+                my %expr = $self->optimize($obj->{if}[$i]{expr});
+                $obj->{if}[$i]{expr} = \%expr;
             }
             if (exists $obj->{else}) {
-                $obj->{else}{block}{code} = {$self->optimize($obj->{else}{block}{code})};
+                my %code = $self->optimize($obj->{else}{block}{code});
+                $obj->{else}{block}{code} = \%code;
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::With') {
             foreach my $i (0 .. $#{$obj->{with}}) {
-                $obj->{with}[$i]{block}{code} = {$self->optimize($obj->{with}[$i]{block}{code})};
-                $obj->{with}[$i]{expr} = {$self->optimize($obj->{with}[$i]{expr})};
+                my %code = $self->optimize($obj->{with}[$i]{block}{code});
+                $obj->{with}[$i]{block}{code} = \%code;
+
+                my %expr = $self->optimize($obj->{with}[$i]{expr});
+                $obj->{with}[$i]{expr} = \%expr;
             }
             if (exists $obj->{else}) {
-                $obj->{else}{block}{code} = {$self->optimize($obj->{else}{block}{code})};
+                my %code = $self->optimize($obj->{else}{block}{code});
+                $obj->{else}{block}{code} = \%code;
             }
         }
-        elsif ($ref eq 'Sidef::Types::Block::ForIn') {
-            foreach my $loop (@{$obj->{loops}}) {
-                $loop->{expr} = {$self->optimize($loop->{expr})};
-            }
-
-            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
-        }
         elsif (   $ref eq 'Sidef::Types::Block::While'
+               or $ref eq 'Sidef::Types::Block::ForIn'
                or $ref eq 'Sidef::Types::Block::ForEach') {
-            $obj->{expr} = {$self->optimize($obj->{expr})};
-            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
+            my %expr = $self->optimize($obj->{expr});
+            $obj->{expr} = \%expr;
+
+            my %code = $self->optimize($obj->{block}{code});
+            $obj->{block}{code} = \%code;
         }
         elsif ($ref eq 'Sidef::Types::Block::CFor') {
             foreach my $i (0 .. $#{$obj->{expr}}) {
-                $obj->{expr}[$i] = {$self->optimize($obj->{expr}[$i])};
+                my %expr = $self->optimize($obj->{expr}[$i]);
+                $obj->{expr}[$i] = \%expr;
             }
-            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
+            my %code = $self->optimize($obj->{block}{code});
+            $obj->{block}{code} = \%code;
         }
         elsif ($ref eq 'Sidef::Variable::NamedParam') {
-            $obj->{value} = [map { {main => [$self->optimize_expr({self => $_})]} } @{$obj->{value}}];
+            $obj->[1] = [map { {main => [$self->optimize_expr({self => $_})]} } @{$obj->[1]}];
         }
         elsif ($ref eq 'Sidef::Meta::PrefixMethod') {
-            $obj->{expr} = {$self->optimize($obj->{expr})};
-        }
-        elsif ($ref eq 'Sidef::Meta::Assert') {
-            $obj->{arg} = {$self->optimize($obj->{arg})};
-        }
-        elsif ($ref eq 'Sidef::Meta::Module') {
-            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
-        }
-        elsif ($ref eq 'Sidef::Meta::Included') {
-            foreach my $info (@{$obj->{included}}) {
-                $info->{ast} = {$self->optimize($info->{ast})};
-            }
-        }
-        elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
-            $obj->{cond}  = {$self->optimize($obj->{cond})};
-            $obj->{true}  = {$self->optimize($obj->{true})};
-            $obj->{false} = {$self->optimize($obj->{false})};
+            my %expr = $self->optimize($obj->{expr});
+            $obj->{expr} = \%expr;
         }
 
         if (not exists($expr->{ind}) and not exists($expr->{call})) {
@@ -994,9 +910,9 @@ package Sidef::Optimizer {
                     foreach my $j (0 .. $#{$call->{arg}}) {
                         my $arg = $call->{arg}[$j];
                         push @{$obj->{call}[$i]{arg}},
-                            ref($arg) eq 'HASH' ? {$self->optimize($arg)}
-                          : ref($arg)           ? $self->optimize_expr({self => $arg})
-                          :                       $arg;
+                            ref($arg) eq 'HASH' ? do { my %arg = $self->optimize($arg); \%arg }
+                          : ref($arg) ? $self->optimize_expr({self => $arg})
+                          :             $arg;
                     }
                 }
 
@@ -1005,9 +921,9 @@ package Sidef::Optimizer {
                     foreach my $j (0 .. $#{$call->{block}}) {
                         my $arg = $call->{block}[$j];
                         push @{$obj->{call}[$i]{block}},
-                            ref $arg eq 'HASH' ? {$self->optimize($arg)}
-                          : ref($arg)          ? $self->optimize_expr({self => $arg})
-                          :                      $arg;
+                            ref $arg eq 'HASH' ? do { my %arg = $self->optimize($arg); \%arg }
+                          : ref($arg) ? $self->optimize_expr({self => $arg})
+                          :             $arg;
                     }
                 }
 
@@ -1052,9 +968,6 @@ package Sidef::Optimizer {
                                 }
 
                                 if ($ok) {
-                                    if (exists $dt_table{$ref_obj}) {
-                                        $obj->{self} = $dt_table{$ref_obj};
-                                    }
                                     $obj->{self} = $obj->{self}->$code(@args);
                                     $ref_obj     = ref($obj->{self});
                                     $optimized   = 1;
